@@ -2,17 +2,14 @@
 
 import re
 
-from lintian_brush import available_lintian_fixers
+from lintian_brush import fixable_lintian_tags
+
+supported_tags = fixable_lintian_tags()
 
 with open("README.md") as f:
     contents = f.read()
 
-fixers = available_lintian_fixers()
-
-tags = set()
-for fixer in fixers:
-    tags.update(fixer.lintian_tags)
-replacement_text = "".join([f"* {tag}\n" for tag in sorted(tags)])
+replacement_text = "".join([f"* {tag}\n" for tag in sorted(supported_tags)])
 
 with open("README.md", "w") as f:
     # TODO(jelmer): Use better sentinels, just in case somebody changes
