@@ -1,16 +1,16 @@
-use breezyshim::branch::{open_containing as open_containing_branch, BranchOpenError};
-use breezyshim::tree::{MutableTree, WorkingTree, WorkingTreeOpenError};
+use breezyshim::branch::open_containing as open_containing_branch;
+use breezyshim::error::Error;
+use breezyshim::tree::MutableTree;
+use breezyshim::workingtree;
 use clap::Parser;
 use debian_changelog::get_maintainer;
 use distro_info::DistroInfo;
 
-use debian_analyzer::svp::{
-    enabled as svp_enabled, load_resume, report_fatal, report_success_debian,
-};
 use debian_analyzer::{get_committer, Certainty};
 use lintian_brush::{ManyResult, OverallError};
 use std::collections::HashMap;
 use std::io::Write as _;
+use std::path::PathBuf;
 
 #[derive(clap::Args, Clone, Debug)]
 #[group()]
@@ -19,8 +19,8 @@ struct FixerArgs {
     fixers: Option<Vec<String>>,
 
     /// Path to fixer scripts
-    #[arg(short, long, default_value_t = lintian_brush::find_fixers_dir().unwrap().display().to_string(), value_name="DIR", help_heading = Some("Fixers"))]
-    fixers_dir: String,
+    #[arg(short, long)]
+    fixers_dir: Option<PathBuf>,
 
     /// Exclude fixers
     #[arg(long, value_name = "EXCLUDE", help_heading = Some("Fixers"))]
@@ -153,7 +153,7 @@ struct Args {
     output: OutputArgs,
 }
 
-fn main() -> Result<(), Box<dyn std::error::Error>> {
+fn main() -> Result<(), i32> {
     let args = Args::parse();
 
     env_logger::builder()
@@ -168,13 +168,23 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
         )
         .init();
 
-    breezyshim::init().unwrap();
+    breezyshim::init();
 
-    let mut fixers: Vec<_> = lintian_brush::available_lintian_fixers(
-        Some(std::path::PathBuf::from(args.fixers.fixers_dir).as_path()),
+    // TODO(jelmer): Allow changing this via arguments
+    let timeout = Some(chrono::Duration::seconds(10));
+
+    let fixers_iter = match lintian_brush::available_lintian_fixers(
+        args.fixers.fixers_dir.as_deref(),
         Some(args.fixers.force_subprocess),
-    )?
-    .collect();
+    ) {
+        Ok(fixers) => fixers,
+        Err(e) => {
+            log::error!("Error loading fixers: {}", e);
+            std::process::exit(1);
+        }
+    };
+
+    let mut fixers: Vec<_> = fixers_iter.collect();
 
     if args.output.list_fixers {
         fixers.sort_by_key(|a| a.name());
@@ -207,11 +217,11 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
                 &url::Url::from_directory_path(&args.output.directory).unwrap(),
             ) {
                 Ok((branch, subpath)) => (branch, subpath),
-                Err(BranchOpenError::NotBranchError(_msg)) => {
+                Err(Error::NotBranchError(_msg, _)) => {
                     log::error!("No version control directory found (e.g. a .git directory).");
                     std::process::exit(1);
                 }
-                Err(BranchOpenError::DependencyNotPresent(name, _reason)) => {
+                Err(Error::DependencyNotPresent(name, _reason)) => {
                     log::error!(
                         "Unable to open branch at {}: missing package {}",
                         args.output.directory.display(),
@@ -219,7 +229,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
                     );
                     std::process::exit(1);
                 }
-                Err(BranchOpenError::Other(err)) => {
+                Err(err) => {
                     log::error!(
                         "Unable to open branch at {}: {}",
                         args.output.directory.display(),
@@ -227,32 +237,41 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
                     );
                     std::process::exit(1);
                 }
-                Err(BranchOpenError::NoColocatedBranchSupport) => {
-                    panic!("NoColocatedBranchSupport should not be returned by open_containing");
+            };
+
+            let td = match tempfile::tempdir() {
+                Ok(td) => td,
+                Err(e) => {
+                    log::error!("Unable to create temporary directory: {}", e);
+                    std::process::exit(1);
                 }
             };
 
-            let td = tempfile::tempdir()?;
-
             // TODO(jelmer): Make a slimmer copy
 
-            let to_dir = branch.controldir().sprout(
+            let to_dir = match branch.controldir().sprout(
                 url::Url::from_directory_path(td.path()).unwrap(),
                 Some(branch.as_ref()),
                 Some(true),
                 Some(branch.format().supports_stacking()),
                 None,
-            );
+            ) {
+                Ok(to_dir) => to_dir,
+                Err(e) => {
+                    log::error!("Unable to create temporary branch: {}", e);
+                    std::process::exit(1);
+                }
+            };
             tempdir = Some(td);
-            (to_dir.open_workingtree()?, subpath)
+            (to_dir.open_workingtree().unwrap(), subpath)
         } else {
-            match WorkingTree::open_containing(&args.output.directory) {
+            match workingtree::open_containing(&args.output.directory) {
                 Ok((wt, subpath)) => (wt, subpath.display().to_string()),
-                Err(WorkingTreeOpenError::NotBranchError(_msg)) => {
+                Err(Error::NotBranchError(_msg, _)) => {
                     log::error!("No version control directory found (e.g. a .git directory).");
                     std::process::exit(1);
                 }
-                Err(WorkingTreeOpenError::DependencyNotPresent(name, _reason)) => {
+                Err(Error::DependencyNotPresent(name, _reason)) => {
                     log::error!(
                         "Unable to open tree at {}: missing package {}",
                         args.output.directory.display(),
@@ -260,7 +279,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
                     );
                     std::process::exit(1);
                 }
-                Err(WorkingTreeOpenError::Other(e)) => {
+                Err(e) => {
                     log::error!(
                         "Unable to open tree at {}: {}",
                         args.output.directory.display(),
@@ -276,6 +295,9 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             println!("Changelog identity: {} <{}>", maintainer, email);
             std::process::exit(0);
         }
+
+        let svp = svp_client::Reporter::new(versions_dict());
+
         let since_revid = wt.last_revision().unwrap();
         if args.fixers.fixers.is_some() || args.fixers.exclude.is_some() {
             let include = args
@@ -356,43 +378,85 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             },
             |s| s.clone(),
         );
+
+        if args.output.verbose {
+            log::info!("Using parameters:");
+            log::info!(" compatibility release: {}", compat_release);
+            log::info!(" minimum certainty: {}", minimum_certainty);
+            if let Some(allow_reformatting) = allow_reformatting {
+                log::info!(" allow reformatting: {}", allow_reformatting);
+            } else {
+                log::info!(" allow reformatting: auto");
+            }
+            if let Some(update_changelog) = update_changelog {
+                log::info!(" update changelog: {}", update_changelog);
+            } else {
+                log::info!(" update changelog: auto");
+            }
+        }
+
         let write_lock = wt.lock_write();
         if debian_analyzer::control_files_in_root(&wt, std::path::Path::new(subpath.as_str())) {
             drop(write_lock);
-            report_fatal(
-                versions_dict(),
+            svp.report_fatal(
                 "control-files-in-root",
                 "control files live in root rather than debian/ (LarstIQ mode)",
                 None,
-                None,
+                Some(false),
             );
         }
 
+        #[cfg(feature = "python")]
+        {
+            // Ensure we can find the lintian_brush.fixer python module
+            let e = pyo3::Python::with_gil(|py| {
+                if let Err(e) = py.import_bound("lintian_brush.fixer") {
+                    Some(e)
+                } else {
+                    None
+                }
+            });
+
+            if let Some(e) = e {
+                drop(write_lock);
+                svp.report_fatal(
+                    "python-import-error",
+                    format!("Error importing lintian_brush.fixer: {}", e).as_str(),
+                    Some("Ensure that the lintian-brush Python package is in Python's sys.path."),
+                    Some(false),
+                );
+            }
+        }
+
+        let preferences = lintian_brush::FixerPreferences {
+            compat_release: Some(compat_release),
+            minimum_certainty: Some(minimum_certainty),
+            allow_reformatting,
+            net_access: Some(!args.output.disable_net_access),
+            opinionated: Some(args.fixers.opinionated),
+            diligence: Some(args.fixers.diligent),
+            trust_package: Some(args.packages.trust),
+        };
+
         let mut overall_result = match lintian_brush::run_lintian_fixers(
             &wt,
             fixers.as_slice(),
             update_changelog.as_ref().map(|b| (|| *b)),
             args.output.verbose,
             None,
-            Some(compat_release.as_str()),
-            Some(minimum_certainty),
-            Some(args.packages.trust),
-            allow_reformatting,
+            &preferences,
             if args.output.disable_inotify {
                 Some(false)
             } else {
                 None
             },
             Some(std::path::Path::new(subpath.as_str())),
-            Some(!args.output.disable_net_access),
-            Some(args.fixers.opinionated),
-            Some(args.fixers.diligent),
             Some("lintian-brush"),
+            timeout,
         ) {
             Err(OverallError::NotDebianPackage(p)) => {
                 drop(write_lock);
-                report_fatal(
-                    versions_dict(),
+                svp.report_fatal(
                     "not-debian-package",
                     format!("{}: Not a Debian package", p.display()).as_str(),
                     None,
@@ -412,8 +476,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             }
             Err(OverallError::ChangelogCreate(e)) => {
                 drop(write_lock);
-                report_fatal(
-                    versions_dict(),
+                svp.report_fatal(
                     "changelog-create-error",
                     format!("Error creating changelog entry: {}", e).as_str(),
                     None,
@@ -422,8 +485,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             }
             Err(OverallError::InvalidChangelog(p, s)) => {
                 drop(write_lock);
-                report_fatal(
-                    versions_dict(),
+                svp.report_fatal(
                     "invalid-changelog",
                     format!("{}: Invalid changelog: {}", p.display(), s).as_str(),
                     None,
@@ -433,18 +495,16 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             #[cfg(feature = "python")]
             Err(OverallError::Python(e)) => {
                 drop(write_lock);
-                report_fatal(
-                    versions_dict(),
+                svp.report_fatal(
                     "python-error",
                     format!("Error running Python: {}", e).as_str(),
                     None,
                     None,
                 );
             }
-            Err(OverallError::TreeError(e)) => {
+            Err(OverallError::BrzError(e)) => {
                 drop(write_lock);
-                report_fatal(
-                    versions_dict(),
+                svp.report_fatal(
                     "internal-error",
                     format!("Tree manipulation error: {}", e).as_str(),
                     None,
@@ -453,18 +513,11 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             }
             Err(OverallError::IoError(e)) => {
                 drop(write_lock);
-                report_fatal(
-                    versions_dict(),
-                    "io-error",
-                    format!("I/O error: {}", e).as_str(),
-                    None,
-                    None,
-                );
+                svp.report_fatal("io-error", format!("I/O error: {}", e).as_str(), None, None);
             }
             Err(OverallError::Other(e)) => {
                 drop(write_lock);
-                report_fatal(
-                    versions_dict(),
+                svp.report_fatal(
                     "other-error",
                     format!("Other error: {}", e).as_str(),
                     None,
@@ -514,10 +567,11 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             log::info!("No changes made.");
         }
         if !overall_result.failed_fixers.is_empty() && !args.output.verbose {
-            log::info!(
-                "Some fixer scripts failed to run: {:?}. Run with --verbose for details.",
-                overall_result.failed_fixers.keys().collect::<Vec<_>>(),
-            );
+            log::info!("Some fixer scripts failed to run:");
+            for (name, reason) in overall_result.failed_fixers.iter() {
+                log::info!("  {}: {}", name, reason);
+            }
+            log::info!("Run with --verbose for details.");
         }
         if !overall_result.formatting_unpreservable.is_empty() && !args.output.verbose {
             log::info!(
@@ -536,22 +590,18 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
                 Box::new(std::io::stdout()),
                 None,
                 None,
-            )?;
+            )
+            .unwrap();
         }
-        if svp_enabled() {
-            if let Some(base) = load_resume() {
-                let base: ManyResult = serde_json::from_value(base)?;
+        if svp.enabled() {
+            if let Some(base) = svp.load_resume::<ManyResult>() {
                 overall_result.success.extend(base.success);
             }
-            let changelog_behaviour = overall_result
-                .changelog_behaviour
-                .as_ref()
-                .map(|b| b.into());
-            report_success_debian(
-                versions_dict(),
+            let changelog_behaviour = overall_result.changelog_behaviour.clone();
+            svp.report_success_debian(
                 Some(overall_result.value()),
-                Some(serde_json::to_value(overall_result)?),
-                changelog_behaviour,
+                Some(overall_result),
+                changelog_behaviour.map(|b| b.into()),
             )
         }
     }
@@ -559,6 +609,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
 }
 
 fn versions_dict() -> HashMap<String, String> {
+    use pyo3::prelude::*;
     let mut ret = HashMap::new();
     ret.insert(
         "lintian-brush".to_string(),
@@ -568,7 +619,7 @@ fn versions_dict() -> HashMap<String, String> {
     ret.insert("breezy".to_string(), breezy_version.to_string());
 
     pyo3::Python::with_gil(|py| {
-        let debmutate = py.import("debmutate").unwrap();
+        let debmutate = py.import_bound("debmutate").unwrap();
         ret.insert(
             "debmutate".to_string(),
             debmutate
@@ -578,7 +629,7 @@ fn versions_dict() -> HashMap<String, String> {
                 .unwrap(),
         );
 
-        let debian = py.import("debian").unwrap();
+        let debian = py.import_bound("debian").unwrap();
         ret.insert(
             "debian".to_string(),
             debian.getattr("__version__").unwrap().extract().unwrap(),
